#ifndef CORE_NUMTYPES_H
#define CORE_NUMTYPES_H


#ifdef __cplusplus

class int16_LE
{
public:
    unsigned char a,b;


    int16_LE() {}

    template<typename T>
    int16_LE(const T &v) { *this=v; }

    short operator=(short p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);

        return (short)(*this);
    }

    operator short() const
    {
        return (short)((((unsigned)a)) +
                      (((int)b)<<8));
    }

};

class int16_BE
{
public:
    unsigned char b,a;

    int16_BE() {}

    template<typename T>
    int16_BE(const T &v) { *this=v; }


    short operator=(short p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        return (short)(*this);
    }

    operator short()  const
    {
        return (short)((((unsigned)a)) +
                      (((int)b)<<8));
    }

};



class int32_LE
{
public:
    unsigned char a,b,c,d;

    int32_LE() {}

    template<typename T>
    int32_LE(const T &v) { *this=v; }

    int operator=(int p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (int)(*this);
    }

    operator int()  const
    {
        return (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((int)d)<<24);
    }

};

class int32_BE
{
public:
    unsigned char d,c,b,a;

    int32_BE() {}

    template<typename T>
    int32_BE(const T &v) { *this=v; }

    int operator=(int p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (int)(*this);
    }

    operator int() const
    {
        return (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((int)d)<<24);
    }
};

class uint16_LE
{
public:
    unsigned char a,b;


    uint16_LE() {}

    template<typename T>
    uint16_LE(const T &v) { *this=v; }


    unsigned short operator=(unsigned short p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);

        return (unsigned short)(*this);
    }

    operator unsigned short()  const
    {
        return (unsigned short)((((unsigned)a)) +
                      (((unsigned)b)<<8));
    }

};

class uint16_BE
{
public:
    unsigned char b,a;

    uint16_BE() {}

    template<typename T>
    uint16_BE(const T &v) { *this=v; }


    unsigned short operator=(unsigned short p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        return (unsigned short)(*this);
    }

    operator unsigned short()  const
    {
        return (unsigned short)((((unsigned)a)) +
                      (((unsigned)b)<<8));
    }

};



class uint32_LE
{
public:
    unsigned char a,b,c,d;

    uint32_LE() {}

    template<typename T>
    uint32_LE(const T &v) { *this=v; }


    unsigned operator=(unsigned p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (unsigned)(*this);
    }

    operator unsigned()  const
    {
        return (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((unsigned)d)<<24);
    }

};

class uint32_BE
{
public:
    unsigned char d,c,b,a;

    uint32_BE() {}

    template<typename T>
    uint32_BE(const T &v) { *this=v; }

    unsigned operator=(unsigned p) 
    {
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (unsigned)(*this);
    }

    operator unsigned()  const
    {
        return (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((unsigned)d)<<24);
    }
};

class float_LE 
{
public:
    unsigned char a,b,c,d;

    float_LE() {}

    template<typename T>
    float_LE(const T &v) { *this=v; }

    float operator=(float _p) 
    {
        unsigned p = *(unsigned *)&_p;
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (float)(*this);
    }

    float operator*=(float v) { *this=(*this)*v; return *this; }

    operator float()  const
    {
        unsigned p=
               (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((unsigned)d)<<24);

        return *(float*)&p;
    }

};

class float_BE 
{
public:
    unsigned char d,c,b,a;

    float_BE() {}

    template<typename T>
    float_BE(const T &v) { *this=v; }

    float operator=(float _p) 
    {
        unsigned p = *(unsigned *)&_p;
        a = (unsigned char)(p&0xFF);
        b = (unsigned char)((p>>8)&0xFF);
        c = (unsigned char)((p>>16)&0xFF);
        d = (unsigned char)((p>>24)&0xFF);
        return (float)(*this);
    }

    float operator*=(float v) { *this=(*this)*v; return *this; }

    operator float()  const
    {
        unsigned p=
               (((unsigned)a))     +
               (((unsigned)b)<<8)  +
               (((unsigned)c)<<16) +
               (((unsigned)d)<<24);

        return *(float*)&p;
    }

};

class int64_LE
{
public:
    int32_LE x,y;

    int64_LE() {}

    template<typename T>
    int64_LE(const T &v) { *this=v; }

    long long operator=(long long p) 
    {
        x=(int)p;
        y=(int)(p>>32);
        return (int)(*this);
    }

    operator long long()  const
    {
        return (long long)(((unsigned)(int)x) + (((unsigned long long)(unsigned)(int)y)<<32));
    }

};

class uint64_LE
{
public:
    uint32_LE x,y;

    uint64_LE() {}

    template<typename T>
    uint64_LE(const T &v) { *this=v; }

    unsigned long long operator=(unsigned long long p) 
    {
        x=(unsigned)p;
        y=(unsigned)(p>>32);
        return (unsigned)(*this);
    }

    operator unsigned long long()  const
    {
        return (unsigned long long)(((unsigned)x) + (((unsigned long long)(unsigned)y)<<32));
    }

};


#endif // __cplusplus

#if defined(__ppc__)
#define int16_BE  short
#define uint16_BE unsigned short
#define int32_BE  int
#define uint32_BE unsigned int
#define float_BE float
#endif

#if defined(__i386__)||defined(_MSC_VER)||defined(WIN32)||defined(__x86_64__)
#define int16_LE  short
#define uint16_LE unsigned short
#define int32_LE  int
#define uint32_LE unsigned int
#define float_LE float
#define int64_LE long long
#define uint64_LE unsigned long long
#endif


#endif
